__title__ = "aamras"
__description__ = "Library for headless browser manipulation"
__url__ = "https://github.com/allekmott/aamras"
__version__ = "0.0.3"
__author__ = "Allek Mott"
__copyright__ = "2020, Allek Mott"
__author_email__ = "allekmott@gmail.com"
__license__ = "GPLv3"
